Strict
Rem
bbdoc: Simple Printing Module
about:
This module allows you to list installed printers, create pages and draw text and images to them. Win32 only.
EndRem
Module sunty.easyprinter

ModuleInfo "Version: 0.04"
ModuleInfo "Authors: SamFisher, wherewolf"
ModuleInfo "DLL by: Christian Klaussner"
ModuleInfo "License: Public Domain"

ModuleInfo "History: fixed some type mismatches -> EP_LoadFont should work now"
ModuleInfo "History: 0.03 added documentation"
ModuleInfo "History: 0.02 fixed EP_DrawBuffer"
ModuleInfo "History: 0.01 EP_DrawBuffer noch nicht funktionstchtig"

Import brl.standardio
Import pub.win32
Import brl.Graphics
Import brl.max2d
Import brl.d3d7max2d

?Not win32
	Throw "this module only works on win32"
?


'internal functions with wrappers for the outside world
Global EP_Initialize_Internal:Int()"Win32"
Global EP_BeginPrint_Internal(index:Int, document$z, orientation:Int, size:Int, quality:Int)"Win32"
Global EP_DrawBuffer_Internal:Int(buffer:Byte Ptr, x:Int, y:Int)"Win32"


Rem
bbdoc: Count Available Printers
about: Counts the number of installed printers. Those can be hardware- as well as virtual devices.
EndRem
Global EP_PrinterCount:Int()"Win32"

Rem
bbdoc: Get Name of Printer
about: #index has to be in range [0, PrinterCount()-1]
EndRem
Global EP_PrinterName:Int Ptr(index:Int)"Win32"

Rem
bbdoc: Close and Send Print Job
returns: true, if successful
about: Sends the job to the printer
EndRem
Global EP_EndPrint:Int()"Win32"

Rem
bbdoc: Begin New Page
returns: true, if successful
about: Begin a new page to print on
EndRem
Global EP_BeginPage:Int()"Win32"

Rem
bbdoc: Close Page
returns: true, if successful
EndRem
Global EP_EndPage:Int()"Win32"

Rem
bbdoc: Get Width of a Page
returns: width of a page in pixels
about:
Get width in pixels. EP_BeginPrint has to be called before, as the width of a page depends on
the printer and format selected.
EndRem
Global EP_PageWidth:Int()"Win32"

Rem
bbdoc: Get Height of a Page
returns: height of a page in pixels
about:
Get height in pixels. EP_BeginPrint has to be called before, as the height of a page depends on
the printer and format selected.
EndRem
Global EP_PageHeight:Int()"Win32"

Rem
bbdoc: Load A Font
returns: font handle to be used for EP_DrawText
about:
Loads a font into memory.

name$ - font name

height - font height in pixels

bold - true for bold font 

italic - true for italic font

underline - true for underlined font
EndRem
Global EP_LoadFont:Int(name$z, height:Int, bold:Int, italic:Int, underline:Int)"Win32"

Rem
bbdoc: Free A Font
returns: true if successful
about: Remove #font from memory.
EndRem
Global EP_FreeFont:Int(font:Int)"Win32"

Rem
bbdoc: Set Color for DrawText
returns: true if successful
about: Sets color for succeeding DrawText operations. Default color is black.
EndRem
Global EP_TextColor:Int(red:Int, green:Int, blue:Int)"Win32"

Rem
bbdoc: Draw #Text
returns: true if successful
EndRem
Global EP_DrawText(text$z, x:Int, y:Int, font:Int)"Win32"

Rem
bbdoc: Load and initialize the DLL.
returns: true if successful
about:
This function must be called previous to using the module.
#mod_dir has to give the directory of your mod folder, e.g. #C:\BlitzMax\mod.
EndRem
Function EP_Initialize(mod_dir$)
	Local DllHandle:Int  = LoadLibraryA(mod_dir+"/sunty.mod/easyprinter.mod/EasyPrinter.dll")
	If DllHandle = 0 Then Throw "could not load EasyPrinter.dll"

	EP_Initialize_Internal = GetProcAddress(DllHandle,"EP_Initialize")
	EP_PrinterCount = GetProcAddress(DllHandle,"EP_PrinterCount")
	EP_PrinterName = GetProcAddress(DllHandle,"EP_PrinterName")
	EP_BeginPrint_Internal = GetProcAddress(DllHandle,"EP_BeginPrint")
	EP_EndPrint = GetProcAddress(DllHandle,"EP_EndPrint")
	EP_BeginPage = GetProcAddress(DllHandle,"EP_BeginPage")
	EP_EndPage = GetProcAddress(DllHandle,"EP_EndPage")
	EP_PageWidth = GetProcAddress(DllHandle,"EP_PageWidth")
	EP_PageHeight = GetProcAddress(DllHandle,"EP_PageHeight")
	EP_LoadFont = GetProcAddress(DllHandle,"EP_LoadFont")
	EP_FreeFont = GetProcAddress(DllHandle,"EP_FreeFont")
	EP_TextColor = GetProcAddress(DllHandle,"EP_TextColor")
	EP_DrawText = GetProcAddress(DllHandle,"EP_DrawText")
	EP_DrawBuffer_Internal = GetProcAddress(DllHandle,"EP_DrawBuffer")

	Return EP_Initialize_Internal()
EndFunction


Rem
bbdoc: Start New Print Job
returns: true, if successful
about:
Starts a new print job. Only one job at a time can be started.

#index - index of printer to be used

#document$ - name of the document to be printed

#orientation - 1 for landscape, 2 for portrait

#size - A2 (1), A3 (2), A4 (3), A5 (4)

#quality - draft (1), low (2), middle (3) or high (4)
EndRem
Function EP_BeginPrint(index:Int, document$z, orientation:Int=0, size:Int=0, quality:Int=0)
	Return EP_BeginPrint_Internal(index, document, orientation, size, quality)
EndFunction

Rem
bbdoc: Print #img at #x/#y. If #img doesn't fit into the current page, it gets cropped.
returns: true if successful
about:
Before using EP_DrawBuffer you must create a graphics object (using Graphics, e.g.).
Note:
EP_DrawBuffer will reset Graphics in order to set the DX7 graphics driver.
It restores all old state known (width, height, depth, hertz, flags), still this is a rather ugly workaround...
EndRem
Function EP_DrawBuffer:Int(img:TImage, x:Int, y:Int)

	'store current graphics state
	Local old_driver:TGraphicsDriver=GetGraphicsDriver()
	Local w=GraphicsWidth(), h=GraphicsHeight(), d=GraphicsDepth(), hertz=GraphicsHertz(), f=DefaultGraphicsFlags()

	'set DX7 driver
	Local dx7_driver:TMax2DDriver=D3D7Max2DDriver()
	If Not dx7_driver Then Throw "couldn't initialize DX7 driver"
	SetGraphicsDriver(dx7_driver) 'to get a D3D7 frame, we need dx7
	Graphics(w, h, d, hertz, f)
	If Not _Max2DDriver Then Throw "setting D3D7Max2DDriver failed"
	
	'get and print frame
	Local frame:TD3D7ImageFrame, surface:IDirectDrawSurface7
	frame = TD3D7ImageFrame(img.Frame(0))
	If Not frame Then Throw "getting image frame failed"
	surface = frame.surface
	If Not surface Then Throw "getting IDirectDrawSurface7 failed"

	Local buf:Byte Ptr[4]
	buf[3] = Byte Ptr(surface) 'that's the only location EP looks at
	Local success = EP_DrawBuffer_Internal(buf, x, y)
	
	'restore driver and graphics
	SetGraphicsDriver(old_driver)
	Graphics(w, h, d, hertz, f)
	Return success
EndFunction